# frozen_string_literal: true

require 'fileutils'
require 'pathname'
require 'fakefs/pry'
require 'fakefs/irb' if Gem::Specification.find_all_by_name("irb").any?
require 'fakefs/base'
require 'fakefs/fake/file'
require 'fakefs/fake/dir'
require 'fakefs/fake/inode'
require 'fakefs/fake/symlink'
require 'fakefs/file_system'
require 'fakefs/fileutils'
require 'fakefs/file'
require 'fakefs/file_test'
require 'fakefs/dir'
require 'fakefs/globber'
require 'fakefs/io'
require 'fakefs/pathname'
require 'fakefs/kernel'
